#include "XmlElement.h"

#include "XmlAttribute.h"

XmlElement::XmlElement(const String& tagName)
    : mTagName(tagName),
      mChildren()
{

}

XmlElement::~XmlElement()
{

}

Ptr<XmlElement> XmlElement::Create(const String& tagName)
{
    return Ptr<XmlElement>::create(tagName);
}

void XmlElement::setTagName(const String& tagName)
{
    mTagName = tagName;
}

void XmlElement::addChild(Ptr<XmlElement> child)
{
    mChildren.push_back(std::move(child));
}

void XmlElement::addAttribute(Ptr<XmlAttribute> attribute)
{
    mAttributes.insert(attribute->getName(), std::move(attribute));
}

void XmlElement::addAttribute(const String& name, const String& value)
{
    auto attr = Ptr<XmlAttribute>::create(name);
    attr->setValue(value);
    addAttribute(std::move(attr));
}

const String& XmlElement::getTagName() const
{
    return mTagName;
}

const String& XmlElement::getText() const
{
    return mText;
}

void XmlElement::setText(const String& text)
{
    mText = text;
}

XmlElement* XmlElement::getFirstChildWithTagName(const String& tag)
{
    for(auto& child : mChildren)
    {
        if (child->getTagName() == tag)
        {
            return child.get();
        }
    }

    return nullptr;
}

bool XmlElement::hasAttribute(const String& attribute) const
{
    return (bool)(getAttribute(attribute));
}

const XmlAttribute* XmlElement::getAttribute(const String& attributeName) const
{
    if (auto iter = mAttributes.find(attributeName); iter != mAttributes.end())
    {
        return (*iter).value().get();
    }
    return nullptr;
}

XmlAttribute* XmlElement::getAttribute(const String& attributeName)
{
    if (auto iter = mAttributes.find(attributeName); iter != mAttributes.end())
    {
        //return (*iter).value().get();
    }
    return nullptr;
}

void XmlElement::forEachAttribute(eachAttrFunc func) const
{
    
}

const Map<String, Ptr<XmlAttribute> >& XmlElement::getAttributes() const
{
    return mAttributes;
}

const Vector<Ptr<XmlElement> >& XmlElement::getChildren() const
{
    return mChildren;
}

String XmlElement::toString(unsigned depth, bool keepInline) const
{
    const auto prefix = String(2*depth, ' ');

    String line_ending = keepInline ? "" : "\n";

    auto content = prefix + "<" + getTagName();
    for (const auto& [key, attribute] : getAttributes())
    {
        content += _s(" ") + attribute->getName() + _s("=\"") + attribute->getValue() + _s("\"");
    }

    const auto num_children = mChildren.size();
    if (num_children == 0 && getText().empty())
    {
        content += _s("/>") + line_ending;
        return content;
    }
    else
    {
        content += _s(">");
    }

    if (!getText().empty())
    {
        content += getText();
    }

    if (num_children>0)
    {
        content += line_ending;
    }

    for(const auto& child : mChildren)
    {
        content += child->toString(depth+1, keepInline);
    }
    if (num_children>0)
    {
        content += prefix;
    }

    content += _s("</") + getTagName() + _s(">") + line_ending;
    return content;
}
