#pragma once

#include "Pointer.h"
#include "String.h"
#include "Map.h"

class XmlAttribute;

class XmlElement
{
public:
    XmlElement(const String& tagName);
    virtual ~XmlElement();

    static Ptr<XmlElement> Create(const String& tagName);

    void addAttribute(Ptr<XmlAttribute> attribute);
    void addAttribute(const String& name, const String& value);
    void addChild(Ptr<XmlElement> child);

    const String& getTagName() const;
    const String& getText() const;

    using eachAttrFunc = std::function<bool(const String& key, const XmlAttribute& attr)>;
    void forEachAttribute(eachAttrFunc func) const;

    const XmlAttribute* getAttribute(const String& attribute) const;
    XmlAttribute* getAttribute(const String& attribute);

    const Map<String, Ptr<XmlAttribute> >& getAttributes() const;

    const Vector<Ptr<XmlElement> >& getChildren() const;

    XmlElement* getFirstChildWithTagName(const String& tag);

    bool hasAttribute(const String& attribute) const;

    void setText(const String& text);
    void setTagName(const String& tagName);

    virtual String toString(unsigned depth = 0, bool keepInline = false) const;

protected:
    String mTagName;
    String mText;

    Map<String, Ptr<XmlAttribute> > mAttributes;
    Vector<Ptr<XmlElement> > mChildren;
};
